/*
 * null_test.c
 */

#include <nexus.h>

#undef PrintHooks
#define Generic_TestParams	\
nexus_mutex_t	mutex;		\
nexus_cond_t	cond;		\
volatile int	done;		\
int		trips;

#include "generic_test.h"

#define Generic_CleanupEpFunc(ep)

#define Generic_UsageString()				\
	pingpong_usage()

#define Generic_Startup()				\
	(0)

#define Generic_ParseOptions(arg_list)			\
	pingpong_parse_options(arg_list)

#define Generic_TestInit(options, test_params)		\
	pingpong_init(options, test_params)

#define Generic_TestRun_BegRep(test_params)

#define Generic_TestRun_Once(test_params, dest)		\
	pingpong_once(test_params, dest)

#define Generic_TestRun_EndRep(test_params)

#define Generic_TestDone(test_params)			\
	pingpong_done(test_params)

#define Generic_Cleanup()

#define Generic_HandlerTable				\
{ NEXUS_HANDLER_TYPE_NON_THREADED, pingpong_handler	}

#define PINGPONG_HANDLER_ID		0
/*
 * int	src
 * int	dst
 * int	seq
 * int	ttl
 */

/*
 *
 */

static void
pingpong_usage()
{
    printf("    PingPong test:\n"
	   "\ttrips=<n>\t\tnumber of round trips\n"
	   "\t\t\t\t\t0 = one way (1)\n");
}

static int
pingpong_parse_options(arg_list_t *arg_list)
{
    int		trips;
    static char	tmp_buf[20];

    parse_option(arg_list, "trips", "%i", "1", (char *) &trips);
    if (trips < 0) trips = 0;
    sprintf(tmp_buf, "trips=%i", trips);
    add_option_str(tmp_buf);
}

static void
pingpong_init(char *options, test_params_t *tp)
{
    if (sscanf(options, "trips=%i", &tp->trips) != 1)
    {
	nexus_printf("pingpong_init(): bad option string '%s'\n",
		     options);
	harness_abort();
    }
    if ((tp->reliable_proto == NEXUS_FALSE) &&
	(tp->trips > 0))
	nexus_printf("pingpong_init(): warning! unreliable proto\n");
    if (nexus_mutex_init(&(tp->mutex), NULL) != NEXUS_SUCCESS) {
	nexus_printf("pingpong_init(): mutex_init failed\n");
	harness_abort();
    }
    if (nexus_cond_init(&(tp->cond), NULL) != NEXUS_SUCCESS) {
	nexus_printf("pingpong_init(): cond_init failed\n");
	harness_abort();
    }
    tp->done = NEXUS_FALSE;
    nexus_endpoint_set_user_pointer(tp->endpoint, (void *) tp);
}

static void
pingpong_once(test_params_t *tp, int dest)
{
    nexus_buffer_t	buffer;
    int			seq, ttl;

    if (nexus_buffer_init(&buffer, 4 * nexus_sizeof_int(1), 0)
	!= NEXUS_SUCCESS) {
	nexus_printf("pingpong_once(): buffer init failed\n");
	harness_abort();
    }
    seq = 23;
    if (tp->trips == 0)
	ttl = -1;
    else
	ttl = (tp->trips * 2) - 1;
    nexus_put_int(&buffer, &(tp->rank), 1);
    nexus_put_int(&buffer, &dest, 1);
    nexus_put_int(&buffer, &seq, 1);
    nexus_put_int(&buffer, &ttl, 1);
    if (nexus_send_rsr(&buffer,
		       tp->sp_list[dest],
		       PINGPONG_HANDLER_ID,
		       NEXUS_TRUE, NEXUS_FALSE)
	!= NEXUS_SUCCESS) {
	nexus_printf("pingpong_once(): send_rsr failed\n");
	harness_abort();
    }
    if (ttl > 0) {
	if (tp->debug_level > 1)
	    nexus_printf("pingpong_once(): %i blocking\n",
			 tp->rank);
	nexus_mutex_lock(&(tp->mutex));
	while (tp->done == NEXUS_FALSE)
	    nexus_cond_wait(&(tp->cond), &(tp->mutex));
	tp->done = NEXUS_FALSE;
	nexus_mutex_unlock(&(tp->mutex));
	if (tp->debug_level > 1)
	    nexus_printf("pingpong_once(): %i awake\n",
			 tp->rank);
    }
}

static void
pingpong_done(test_params_t *tp)
{
    nexus_mutex_destroy(&(tp->mutex));
    nexus_cond_destroy(&(tp->cond));
}

/*
 *
 */
static void
pingpong_handler(nexus_endpoint_t *endpoint,
		 nexus_buffer_t *buffer,
		 nexus_bool_t is_non_threaded_handler)
{
    nexus_buffer_t	reply_buf;
    test_params_t	*tp;
    int			src, dst, seq, ttl;

    tp = (test_params_t *) nexus_endpoint_get_user_pointer(endpoint);
    nexus_get_int(buffer, &src, 1);
    nexus_get_int(buffer, &dst, 1);
    nexus_get_int(buffer, &seq, 1);
    nexus_get_int(buffer, &ttl, 1);
    if (dst != tp->rank)
	nexus_printf("pingpong_handler(): %i rcvd from %i, "
		     "dst=%i\n", tp->rank, dst, src);
    if (ttl == -1) {
	if (tp->debug_level > 1)
	    nexus_printf("pingpong_handler(): %i rcvd from %i, "
			 "one way\n", dst, src);
    } else if (ttl > 0) {
	if (tp->debug_level == 3)
	    nexus_printf("pingpong_handler(): %i rcvd from %i, "
			 "replying\n", dst, src);
	if (nexus_buffer_init(&reply_buf, 4 * nexus_sizeof_int(1), 0)
	    != NEXUS_SUCCESS) {
	    nexus_printf("pingpong_handler(): buffer_init failed\n");
	    harness_abort();
	}
	ttl--;
	nexus_put_int(&reply_buf, &dst, 1);
	nexus_put_int(&reply_buf, &src, 1);
	nexus_put_int(&reply_buf, &seq, 1);
	nexus_put_int(&reply_buf, &ttl, 1);
	if (nexus_send_rsr(&reply_buf,
			   tp->sp_list[src],
			   PINGPONG_HANDLER_ID,
			   NEXUS_TRUE, is_non_threaded_handler)
	    != NEXUS_SUCCESS) {
	    nexus_printf("pingpong_handler(): send_rsr failed\n");
	    harness_abort();
	}
    } else {
	if (tp->debug_level > 1)
	    nexus_printf("pingpong_handler(): %i rcvd from %i, "
			 "signalling\n", dst, src);
	nexus_mutex_lock(&(tp->mutex));
	tp->done = NEXUS_TRUE;
	nexus_cond_signal(&(tp->cond));
	nexus_mutex_unlock(&(tp->mutex));
    }
}
	    
#include "generic_test.c"
