/**********************************************************************
$Id: nexus_utp_main.c,v 1.4 1997/10/21 14:40:43 freedman Exp $

$Source: /nfs/globus1/src/master/nexus_utp/nexus_utp_main.c,v $

$Log: nexus_utp_main.c,v $
Revision 1.4  1997/10/21 14:40:43  freedman
Finishing renaming of defines and variables.

Revision 1.3  1997/10/21 02:37:36  freedman
Changed prefixes of global variables to NEXUS_UTP_.

Revision 1.2  1997/10/20 23:57:02  freedman
Started renaming of functions from UTP_ to nexus_utp_

Revision 1.1  1997/07/23 22:04:09  tuecke
First pass at integrating Dave Kohr UTP timing package into nexus

Revision 1.6  1995/11/28 21:51:01  tuecke
Added SGI Irix support

Revision 1.5  1995/02/14  00:36:00  patton
Support for dynamic context loading on IBM AIX using load and by
cloning the data segment.

Revision 1.4  1994/11/08  06:36:22  kohr
Port to Paragon using dclock().

Revision 1.3  1994/11/07  15:38:14  patton
added functionality for Solaris high-resolution timer..

Revision 1.2  1994/11/03  19:52:40  kohr
Move current source from RCS to CVS control.

Revision 1.1  1994/10/25  20:42:07  tuecke
An initial version of Dave Kohr's timing package

Public functions for the Unnamed Timing Package (UTP).
**********************************************************************/

/*
Pre-CVS version information:

Revision 1.9  1994/11/03  02:24:36  kohr
Added support for key/value pairs with UTP_set_attribute().

Revision 1.8  1994/11/01  02:29:31  kohr
Added timer naming and disabling/enabling; other clean-up.

Revision 1.7  1994/10/25  20:02:22  kohr
Use external assembler routine for RS6000 clock.

Revision 1.6  1994/10/25  06:27:17  kohr
Use gettimer() on RS6000.

Revision 1.5  1994/10/11  00:17:14  kohr
Removed debugging statement.

Revision 1.4  1994/10/10  23:53:56  kohr
Fixed big bug in UTP_set_filename().

Revision 1.3  1994/10/07  19:25:46  kohr
Minor bug-fix in UTP_shutdown.

Revision 1.2  1994/10/07  03:56:57  kohr
Added UTP_reset_timer(), UTP_get_accum_time(), UTP_timeToDouble().

Revision 1.1  1994/09/14  06:04:16  kohr
Initial revision
*/

#ifdef RS6000		/* Stooopid IBM header files require this to get
			   the right stuff from <pwd.h>. */
#define _ALL_SOURCE
#endif

#include <pwd.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <unistd.h>
#include <sys/param.h>
#include <sys/types.h>

#ifdef SOLHR
#include <netdb.h>
#endif


#include "nexus_utp.h"
#include "nexus_utp_private.h"

/**********************************************************************
Public UTP functions.
**********************************************************************/

int nexus_utp_init(unsigned numTimers, int mode)
{
	int i;
	int sharingMode = mode & NEXUS_UTP_MODE_SHARING_FIELD;
	struct passwd *passwdEntry;
	struct passwd pE;
	char hostnameBuff[MAXHOSTNAMELEN];
	time_t theTime;
	char theTimeStr[27];

	if (sharingMode == NEXUS_UTP_MODE_PRIVATE) {
		nexus_utp_warn("nexus_utp_init: mode NEXUS_UTP_MODE_PRIVATE not yet "
			 "implemented");
		return 1;
	} else if (sharingMode != NEXUS_UTP_MODE_SHARED) {
		nexus_utp_warn("nexus_utp_init: invalid sharing mode %#X specified",
			 sharingMode);
		return 1;
	}

	if (!(nexus_utp_timers = malloc(numTimers * sizeof(nexus_utp_Timer_t)))) {
		nexus_utp_warn("nexus_utp_init: malloc() failed for timers");
		nexus_utp_numTimers = 0;
		return 1;
	}

	nexus_utp_numTimers = numTimers;

				/* This loop not only initializes every
				   field; by "touching" everything, it also
				   makes sure it is paged in. */
	for (i = 0; i < numTimers; i++) {
		nexus_utp_timers[i].state = NEXUS_UTP_TIMER_STOPPED;
		nexus_utp_timeZero(&nexus_utp_timers[i].startTime);
		nexus_utp_timeZero(&nexus_utp_timers[i].accumulatedTime);
		nexus_utp_timers[i].numEvents = 0;
				/* If name is still NULL when
				   nexus_utp_shutdown() is called,
				   nexus_utp_DEFAULT_TIMER_NAME is printed. */
		nexus_utp_timers[i].name = NULL;
	}

	nexus_utp_set_filename(NEXUS_UTP_DEFAULT_OUT_FILE_NAME);

				/* Make key/value linked list empty. */
	nexus_utp_attributes = NULL;

				/* Set attributes for "user", "hostname",
				   and "timestamp". */

	passwdEntry = getpwuid(getuid());
	if (!passwdEntry) {
		nexus_utp_warn("nexus_utp_init: getpwuid() failed; system error is "
		         "\"%s\"", sys_errlist[errno]);
		return 1;
	}
	nexus_utp_set_attribute("user", "", passwdEntry->pw_name);

	if (gethostname(hostnameBuff, MAXHOSTNAMELEN)) {
		nexus_utp_warn("nexus_utp_init: gethostname() failed; system error is "
		         "\"%s\"", sys_errlist[errno]);
		return 1;
	}
	nexus_utp_set_attribute("hostname", "", hostnameBuff);

	if ((theTime = time(NULL)) == (time_t) -1) {
		nexus_utp_warn("nexus_utp_init: time(3) failed; system error is "
		         "\"%s\"", sys_errlist[errno]);
		return 1;
	}
	strcpy(theTimeStr, ctime(&theTime));
				/* Remove trailing '\n'. */
	theTimeStr[strlen(theTimeStr) - 1] = 0;
	nexus_utp_set_attribute("timestamp", "", theTimeStr);

	return 0;
}

void nexus_utp_set_filename(const char *outFilename)
{
	if (nexus_utp_outFilename)
		free(nexus_utp_outFilename);
	if (!(nexus_utp_outFilename = malloc(strlen(outFilename) + 1))) { 
		nexus_utp_warn("nexus_utp_set_filename : malloc() failed");
		return;
	}
	strcpy(nexus_utp_outFilename, outFilename);
}

void
nexus_utp_set_attribute(const char *keyStr, const char *keyArg,
		  const char *valueStr, ...)
{
	nexus_utp_Attribute_t *newAttr;	/* New key/value pair. */
	nexus_utp_Attribute_t **lastNext;	/* Final "next" pointer in list. */
	char *newKey = malloc(sizeof(char) * NEXUS_UTP_MAX_NAME_LENGTH);
	char *newValue = malloc(sizeof(char) * NEXUS_UTP_MAX_NAME_LENGTH);
	nexus_utp_Attribute_t *oldAttr;	/* Old attr. with same key. */
	va_list ap;

	if (!newKey || !newValue) {
		nexus_utp_warn("nexus_utp_set_attribute: malloc() failed");
		return;
	}
	sprintf(newKey, keyStr, keyArg);	/* Compute new key. */
	va_start(ap, valueStr);			/* Compute new value. */
	vsprintf(newValue, valueStr, ap);

	newAttr = nexus_utp_attributes;
	lastNext = &nexus_utp_attributes;
					/* Search for same key in list. */
	while (newAttr && strcmp(newAttr->key, newKey) != 0) {
		lastNext = &newAttr->next;
		newAttr = newAttr->next;
	}

	if (newAttr) {			/* Key already exists in list. */
					/* Just replace the old value. */
		strcpy(newAttr->value, newValue);
		free(newKey);
		free(newValue);
		return;
	}

				/* Add new key/value pair to list. */
	*lastNext = newAttr = malloc(sizeof(nexus_utp_Attribute_t));
	if (!newAttr) {
		nexus_utp_warn("nexus_utp_set_attribute: malloc() failed");
		return;
	}
	newAttr->key = newKey;
	newAttr->value = newValue;
	newAttr->next = NULL;
}


/**********************************************************************
Timer state transition diagram.  If timer is in "Current State", and 
"State Change Function" is called, result is either the indicated new state
or "Invalid" (in which case nexus_utp_warn() is called).

				State Change Function
Current
State		start()	   stop()     reset()    disable()   enable()
-------		-------	   ------     -------    ---------   ---------
STOPPED		RUNNING    Invalid    STOPPED    DISABLED    STOPPED    

RUNNING		Invalid    STOPPED    STOPPED    Invalid     RUNNING    

DISABLED	DISABLED   DISABLED   DISABLED   DISABLED    STOPPED    
**********************************************************************/

void nexus_utp_start_timer(unsigned timerNumber)
{
	nexus_utp_Timer_t *theTimer;

		/* Don't compile run-time checks into fully-optimized
		   version of the library. */
#ifndef NDEBUG
	if (timerNumber >= nexus_utp_numTimers) {
		nexus_utp_warn("nexus_utp_start_timer: illegal timer %d",
			 (int) timerNumber);
		return;
	}
#endif /* #ifndef NDEBUG */

	theTimer = &nexus_utp_timers[timerNumber];
	if (theTimer->state == NEXUS_UTP_TIMER_DISABLED)
		return;

#ifndef NDEBUG
	if (theTimer->state == NEXUS_UTP_TIMER_RUNNING) {
		nexus_utp_warn("nexus_utp_start_timer: timer %d already running",
			 (int) timerNumber);
		return;
	}
#endif /* #ifndef NDEBUG */

	theTimer->state = NEXUS_UTP_TIMER_RUNNING;
	nexus_utp_readTime(&theTimer->startTime);
}

void nexus_utp_stop_timer(unsigned timerNumber)
{
	nexus_utp_Timer_t *theTimer;
	nexus_utp_TimeValue_t stopTime, elapsedTime;

#ifndef NDEBUG
	if (timerNumber >= nexus_utp_numTimers) {
		nexus_utp_warn("nexus_utp_stop_timer: illegal timer %d",
			 (int) timerNumber);
		return;
	}
#endif /* #ifndef NDEBUG */

	theTimer = &nexus_utp_timers[timerNumber];
	if (theTimer->state == NEXUS_UTP_TIMER_DISABLED)
		return;

#ifndef NDEBUG
	if (theTimer->state == NEXUS_UTP_TIMER_STOPPED) {
		nexus_utp_warn("nexus_utp_stop_timer: timer %d not running",
			 (int) timerNumber);
		return;
	}
#endif /* #ifndef NDEBUG */

	theTimer->state = NEXUS_UTP_TIMER_STOPPED;
	nexus_utp_readTime(&stopTime);
	nexus_utp_timeDifference(&elapsedTime, &theTimer->startTime, &stopTime);
	nexus_utp_timeAdd(&theTimer->accumulatedTime, &elapsedTime);
	theTimer->numEvents++;
}

void nexus_utp_reset_timer(unsigned timerNumber)
{
	nexus_utp_Timer_t *theTimer;

#ifndef NDEBUG
	if (timerNumber >= nexus_utp_numTimers) {
		nexus_utp_warn("nexus_utp_reset_timer: illegal timer %d",
			 (int) timerNumber);
		return;
	}
#endif /* #ifndef NDEBUG */

	theTimer = &nexus_utp_timers[timerNumber];
	if (theTimer->state == NEXUS_UTP_TIMER_DISABLED)
		return;

	theTimer->state = NEXUS_UTP_TIMER_STOPPED;
	nexus_utp_timeZero(&theTimer->startTime);
	nexus_utp_timeZero(&theTimer->accumulatedTime);
	theTimer->numEvents = 0;
}

void nexus_utp_disable_timer(unsigned timerNumber)
{
	nexus_utp_Timer_t *theTimer;

#ifndef NDEBUG
	if (timerNumber >= nexus_utp_numTimers) {
		nexus_utp_warn("nexus_utp_disable_timer: illegal timer %d",
			 (int) timerNumber);
		return;
	}
#endif /* #ifndef NDEBUG */

	theTimer = &nexus_utp_timers[timerNumber];

#ifndef NDEBUG
	if (theTimer->state == NEXUS_UTP_TIMER_RUNNING) {
		nexus_utp_warn("nexus_utp_disable_timer: timer %d is running",
			 (int) timerNumber);
		return;
	}
#endif /* #ifndef NDEBUG */

	theTimer->state = NEXUS_UTP_TIMER_DISABLED;
}

void nexus_utp_enable_timer(unsigned timerNumber)
{
	nexus_utp_Timer_t *theTimer;

#ifndef NDEBUG
	if (timerNumber >= nexus_utp_numTimers) {
		nexus_utp_warn("nexus_utp_disable_timer: illegal timer %d",
			 (int) timerNumber);
		return;
	}
#endif /* #ifndef NDEBUG */

	theTimer = &nexus_utp_timers[timerNumber];

			/* If STOPPED or RUNNING, already enabled.
			   Otherwise, can only get to DISABLED from
			   STOPPED, so go there. */
	if (theTimer->state == NEXUS_UTP_TIMER_DISABLED)
		theTimer->state = NEXUS_UTP_TIMER_STOPPED;
}

void nexus_utp_disable_all_timers(void)
{
	int i;

	for (i = 0; i < nexus_utp_numTimers; i++)
		nexus_utp_disable_timer(i);
}

void nexus_utp_enable_all_timers(void)
{
	int i;

	for (i = 0; i < nexus_utp_numTimers; i++)
		nexus_utp_enable_timer(i);
}

const char *
nexus_utp_name_timer(unsigned timerNumber, const char *nameStr, ...)
{
	char *name;
	va_list ap;

#ifndef NDEBUG
	if (timerNumber >= nexus_utp_numTimers) {
		nexus_utp_warn("nexus_utp_name_timer: illegal timer %d",
			 (int) timerNumber);
		return NULL;
	}
#endif /* #ifndef NDEBUG */
	if (!(name = malloc(sizeof(char) * NEXUS_UTP_MAX_NAME_LENGTH))) {
		nexus_utp_warn("nexus_utp_name_timer: malloc() failed");
		return NULL;
	}

	va_start(ap, nameStr);
	vsprintf(name, nameStr, ap);
	nexus_utp_timers[timerNumber].name = name;

	return name;
}

void
nexus_utp_get_accum_time(unsigned timerNumber, double *time, int *precision)
{
	nexus_utp_Timer_t *theTimer;

#ifndef NDEBUG
	if (timerNumber >= nexus_utp_numTimers) {
		nexus_utp_warn("nexus_utp_get_accum_time: illegal timer %d",
			 (int) timerNumber);
		return;
	}
#endif /* #ifndef NDEBUG */

	theTimer = &nexus_utp_timers[timerNumber];

	nexus_utp_timeToDouble(time, precision, &theTimer->accumulatedTime);
}

void nexus_utp_shutdown(void)
{
	FILE *outFile;
	int i;
	char buff[100];
	nexus_utp_Attribute_t *curAttr;	/* Current key/value to print. */

	if (!(outFile = fopen(nexus_utp_outFilename, "w"))) {
		nexus_utp_warn("nexus_utp_shutdown: fopen() on file \"%s\" failed",
			 nexus_utp_outFilename);
		nexus_utp_warn("nexus_utp_shutdown: system error message is \"%s\"",
			 sys_errlist[errno]);
		return;
	}

	curAttr = nexus_utp_attributes;
	while (curAttr) {
		nexus_utp_Attribute_t *attr;		/* Temporary. */

		fprintf(outFile, "attribute\t\"%s\"\t\"%s\"\n",
			curAttr->key, curAttr->value);
		attr = curAttr;
		curAttr = curAttr->next;
		free(attr);
	}
	nexus_utp_attributes = NULL;

/*
More header stuff should be printed eventually....
*/

	for (i = 0; i < nexus_utp_numTimers; i++) {
		const char *name = nexus_utp_timers[i].name;

		if (nexus_utp_timers[i].state == NEXUS_UTP_TIMER_RUNNING) {
			nexus_utp_warn("nexus_utp_shutdown: timer %d still running",
				 i);
			nexus_utp_stop_timer(i);
		}

		nexus_utp_timeToString(buff, &nexus_utp_timers[i].accumulatedTime);

		fprintf(outFile, "timer\t%u\t%s\t%lu\t\"%s\"\n",
			(unsigned) i,
			buff,
			nexus_utp_timers[i].numEvents,
			name ? name : NEXUS_UTP_DEFAULT_TIMER_NAME);
	}

	if (fclose(outFile)) {
		nexus_utp_warn("nexus_utp_shutdown: fclose() on file \"%s\" failed",
			 nexus_utp_outFilename);
		nexus_utp_warn("nexus_utp_shutdown: system error message is \"%s\"",
			 sys_errlist[errno]);
		return;
	}

	for (i = 0; i < nexus_utp_numTimers; i++)
		if (nexus_utp_timers[i].name)
			free(nexus_utp_timers[i].name);
	free(nexus_utp_timers);
}


/**********************************************************************
Private UTP functions.
**********************************************************************/

			/* Use external assembler routine for RS6000 by
			   default, unless RS6000_GETTIMER is defined. */
#if !defined(RS6000) || (defined(RS6000) && defined(RS6000_GETTIMER))

void
nexus_utp_readTime(nexus_utp_TimeValue_t *tv)
{
#ifdef UNIX
#ifdef BSDGETTIMEOFDAY
	int flag = BSDgettimeofday(tv, NULL);
#else
	int flag = gettimeofday(tv, NULL);
#endif

#ifdef DEBUG
	if (flag) {
		nexus_utp_warn("nexus_utp_readTime: gettimeofday() failed");
		nexus_utp_warn("nexus_utp_readTime: system error message is \"%s\"",
			 sys_errlist[errno]);
	}
#endif /* #ifdef DEBUG */
#endif /* #ifdef UNIX */


#ifdef SOLHR
	*tv = gethrtime();
#endif

#ifdef RS6000

/*
The AIX 3.2.5 on-line "info" about gettimer() presents a totally wrong
calling interface for gettimer().  The code below is based on the example
on p. 59 of the "Optimizing and Tuning Guide for FORTRAN, C, and C++", and
seems to work just fine.
*/

	int flag = gettimer(TIMEOFDAY, tv);

#ifdef DEBUG
	if (flag) {
		nexus_utp_warn("nexus_utp_readTime: gettimer() failed");
		nexus_utp_warn("nexus_utp_readTime: system error message is \"%s\"",
			 sys_errlist[errno]);
	}
#endif /* #ifdef DEBUG */

#endif /* #ifdef RS6000 */

#ifdef PARAGON
	*tv = dclock();
#endif	/* #ifdef PARAGON */
}

#endif	/* #if !defined(RS6000) || .... */


void
nexus_utp_timeDifference(nexus_utp_TimeValue_t *diff, const nexus_utp_TimeValue_t *start,
		   const nexus_utp_TimeValue_t *end)
{
#ifdef UNIX
	diff->tv_sec = end->tv_sec - start->tv_sec;

	if (end->tv_usec < start->tv_usec) {
		diff->tv_sec--;		/* "Borrow" from seconds. */
		diff->tv_usec = 1000000 + end->tv_usec - start->tv_usec;
	} else
		diff->tv_usec = end->tv_usec - start->tv_usec;
#endif /* #ifdef UNIX */

#ifdef RS6000
	diff->tv_sec = end->tv_sec - start->tv_sec;

	if (end->tv_nsec < start->tv_nsec) {
		diff->tv_sec--;		/* "Borrow" from seconds. */
		diff->tv_nsec = 1000000000 + end->tv_nsec - start->tv_nsec;
	} else
		diff->tv_nsec = end->tv_nsec - start->tv_nsec;
#endif /* #ifdef RS6000 */

#ifdef SOLHR
	*diff = *end - *start;
#endif

#ifdef PARAGON
	*diff = *end - *start;
#endif	/* #ifdef PARAGON */
}


void
nexus_utp_timeAdd(nexus_utp_TimeValue_t *oldElapsed,
	    const nexus_utp_TimeValue_t *newElapsed)
{
#ifdef UNIX
	oldElapsed->tv_usec += newElapsed->tv_usec;

	if (oldElapsed->tv_usec >= 1000000) {
		oldElapsed->tv_sec++;	/* "Carry" into seconds. */
		oldElapsed->tv_usec -= 1000000;
	}

	oldElapsed->tv_sec += newElapsed->tv_sec;
#endif /* #ifdef UNIX */

#ifdef RS6000
	oldElapsed->tv_nsec += newElapsed->tv_nsec;

	if (oldElapsed->tv_nsec >= 1000000000) {
		oldElapsed->tv_sec++;	/* "Carry" into seconds. */
		oldElapsed->tv_nsec -= 1000000000;
	}

	oldElapsed->tv_sec += newElapsed->tv_sec;
#endif /* #ifdef RS6000 */

#ifdef SOLHR
	*oldElapsed += *newElapsed;
#endif

#ifdef PARAGON
	*oldElapsed += *newElapsed;
#endif	/* #ifdef PARAGON */
}


void
nexus_utp_timeZero(nexus_utp_TimeValue_t *tv)
{
#ifdef UNIX
	tv->tv_sec = 0;
	tv->tv_usec = 0;
#endif /* #ifdef UNIX */

#ifdef RS6000
	tv->tv_sec = 0;
	tv->tv_nsec = 0;
#endif /* #ifdef RS6000 */

#ifdef SOLHR
	*tv = 0;
#endif

#ifdef PARAGON
	*tv = 0.0;
#endif	/* #ifdef PARAGON */
}


void
nexus_utp_timeToDouble(double *time, int *precision, const nexus_utp_TimeValue_t *tv)
{
#ifdef UNIX
	*time = (double) tv->tv_sec + ((double) tv->tv_usec) / 1000000.0;
	*precision = 6;
#endif /* #ifdef UNIX */

#ifdef RS6000
	*time = (double) tv->tv_sec + ((double) tv->tv_nsec) /
		1000000000.0;
	*precision = 7;
#endif /* #ifdef RS6000 */

#ifdef SOLHR
	*time = ((double)*tv) / 1000000000.0;
	*precision = 6;
#endif

#ifdef PARAGON
	*time = *tv;
	*precision = 7;
#endif	/* #ifdef PARAGON */
}


void
nexus_utp_timeToString(char timeString[], const nexus_utp_TimeValue_t *tv)
{
	double tvAsDouble;
	int precision;

	nexus_utp_timeToDouble(&tvAsDouble, &precision, tv);

	sprintf(timeString, "%.*lf", precision, tvAsDouble);
}


void
nexus_utp_warn(const char *messageStr, ...)
{
	va_list ap;

	va_start(ap, messageStr);

	vfprintf(stderr, messageStr, ap);
	fprintf(stderr, "\n");
	fflush(stderr);		/* Just to be safe. */
}


/**********************************************************************
Private UTP globals.
**********************************************************************/

char *nexus_utp_outFilename = NULL;
unsigned nexus_utp_numTimers = 0;
nexus_utp_Timer_t *nexus_utp_timers = NULL;
nexus_utp_Attribute_t *nexus_utp_attributes;
