/******************************************************************************
CIF_Listener.H

Description:

  Virtual listener class - this class defines the interface to a set of
  listener classes which are used to establish unicast connections.

CVS Information:

  $Source$
  $Date$
  $Revision$
  $State$
  $Author$
******************************************************************************/

#if !defined(CIF_INCLUDE_CIF_LISTENER)
#define CIF_INCLUDE_CIF_LISTENER 1

/******************************************************************************
			     Include header files
******************************************************************************/
#include "CIF.H"


/******************************************************************************
			       Class prototypes
******************************************************************************/
class CIF_Connection;


/******************************************************************************
			   Define CIF_Listener class
******************************************************************************/
class
CIF_Listener
{
    /* CIF_Listener::open()
     *
     * Start listening for connection requests.  If a port number is specified,
     * the listener will be established on that port; otherwise, a port will be
     * assigned by the operating system.
     */
    public: virtual
    CIF_Bool
    open(
	ARG_IN unsigned short		port = 0) = 0;

    /*
     * CIF_Listener::get_Connection()
     *
     * Returns the next established connection.  If no connection is
     * established, this function blocks until such time as one is established
     * or the specified time out elapses.
     */
    public: virtual
    CIF_Connection *
    get_Connection(
	ARG_IN int			timeout = -1) = 0;

    /*
     * CIF_Listener::get_URL()
     *
     * Returns the full URL of the listener
     */
    public: virtual
    char *
    get_URL() = 0;

    /*
     * CIF_Listener::close()
     *
     * Stop listening for connection requests
     * 
     */
    public: virtual
    CIF_Bool
    close() = 0;

    public: virtual
    ~CIF_Listener()
    {
    }
};

#endif
