/******************************************************************************
Nexus_Endpoint.H

Description:

  Classes to provide simple Nexus endpoint functionality

CVS Information:

  $Source$
  $Date$
  $Revision$
  $State$
  $Author$
******************************************************************************/

#if !defined(NEXUS_INCLUDE_NEXUS_ENDPOINT)
#define NEXUS_INCLUDE_NEXUS_ENDPOINT 1

/******************************************************************************
			     Include header files
******************************************************************************/
#include "Nexus.H"

#include "globus_common.h"
#include "nexus.h"


static const int Nexus_Endpoint_l_handle_Message_HID = 0;

/******************************************************************************
			       Class prototypes
******************************************************************************/
class Nexus_Receive_Buffer;


/******************************************************************************
			  Define Nexus_Endpoint class
******************************************************************************/
class
Nexus_Endpoint
{
    /**************************************************************************
				    API methods
    **************************************************************************/

    /*
     * Nexus_Endpoint::Nexus_Endpoint()
     */
    public:
    Nexus_Endpoint();

    /*
     * Nexus_Endpoint::activate()
     */
    public:
    Nexus_Bool
    activate(
	ARG_IN nexus_proto_type_t	proto_type = NEXUS_PROTO_TYPE_ALL,
	ARG_IN void *			proto_info_ptr = GLOBUS_NULL,
	ARG_IN int			proto_info_size = 0);

    /*
     * Nexus_Endpoint::receive()
     */
    public:
    Nexus_Bool
    receive(
	ARG_OUT Nexus_Receive_Buffer &  buffer,
	ARG_IN const long		timeout = -1);

    /*
     * Nexus_Endpoint::poll()
     */
    public:
    Nexus_Bool
    poll();
    
    /*
     * Nexus_Endpoint::deactivate()
     */
    public:
    Nexus_Bool
    deactivate();
    
    /*
     * Nexus_Endpoint::~Nexus_Endpoint()
     */
    public:
    ~Nexus_Endpoint();
    

    /**************************************************************************
		       Internal methods and data structures
    **************************************************************************/

    /*
     * Objects which need access to our private methods
     */
    friend class Nexus_Startpoint;
    friend class Nexus_Listener;

    /*
     * State of the endpoint object
     */
    enum Object_State
    {
	OBJECT_STATE_ACTIVE,
	OBJECT_STATE_INACTIVE,
	OBJECT_STATE_DELETED
    };

    /*
     * Pending error
     */
    enum Pending_Error
    {
	PENDING_ERROR_NONE,
	PENDING_ERROR_OPEN,
	PENDING_ERROR_MEM,
	PENDING_ERROR_PROTO
    };

    /*
     * Nexus_Endpoint::l_deactivate()
     */
    private:
    Nexus_Bool
    l_deactivate();
    
    /*
     * Nexus_Endpoint::bind_Startpoint()
     */
    private:
    Nexus_Bool
    bind_Startpoint(
	ARG_INOUT nexus_startpoint_t *	sp);

    /*
     * Nexus_Endpoint::verify_Object_State()
     */
    private:
    Nexus::Status
    verify_Object_State(
	ARG_IN Object_State		desired_state);

    /*
     * Nexus_Endpoint::bind_detect_Pending_Error()
     */
    private:
    Nexus::Status
    detect_Pending_Error();

    /*
     * Nexus_Endpoint::handle_Message()
     */
    private:
    static void
    handle_Message(
	nexus_endpoint_t *		ep,
	nexus_buffer_t *		buf,
	int				hid);

    /*
     * Nexus_Endpoint::handle_Unknown()
     */
    private:
    static void
    handle_Unknown(
    nexus_endpoint_t *			ep,
    nexus_buffer_t *			buf,
    int					hid);

    /*
     * Internal data members
     */
    private:
    
    /* mutex to protect endpoint data structures */
    nexus_mutex_t			mutex;

    /* condition variable to signal that a new message is waiting to be
       received */
    nexus_cond_t			cond;

    /* endpoint for receiving connection establishment information */
    nexus_endpoint_t			ep;

    /* message queue */
    globus_fifo_t			queue;

    /* state of the endpoint object */
    enum Object_State			object_state;

    /* endpoint error state */
    enum Pending_Error			pending_error;
};

#endif /* NEXUS_INCLUDE_NEXUS_ENDPOINT */
