/******************************************************************************
Nexus_Send_Buffer.C

Description:

  Classes to provide simple Nexus send buffer functionality

CVS Information:

  $Source$
  $Date$
  $Revision$
  $State$
  $Author$
******************************************************************************/

/******************************************************************************
			     Include header files
******************************************************************************/
#include "Nexus_internal.H"
#include "Nexus_Monitor.H"


/******************************************************************************
			 Nexus_Send_Buffer API methods
******************************************************************************/

Nexus_Size
Nexus_Send_Buffer::sizeof_Byte(
    unsigned				count)
{
    return (Nexus_Size) nexus_sizeof_byte(count);
}


Nexus_Size
Nexus_Send_Buffer::sizeof_Char(
    unsigned				count)
{
    return (Nexus_Size) nexus_sizeof_char(count);
}


Nexus_Size
Nexus_Send_Buffer::sizeof_Int(
    unsigned				count)
{
    return (Nexus_Size) nexus_sizeof_int(count);
}


Nexus_Size
Nexus_Send_Buffer::sizeof_Startpoint(
    Nexus_Startpoint *			startpoint,
    unsigned				count)
{
    Nexus_Size				size;
    int					i;
    
    size = 0;
    for (i = 0; i < count; i++)
    {
	size += startpoint[i].sizeof_Internal_Startpoint();
    }

    return size;
}


Nexus_Send_Buffer::Nexus_Send_Buffer()
{
    object_state = OBJECT_STATE_INACTIVE;
    Nexus::status = Nexus::STATUS_SUCCESS;
}


Nexus_Bool
Nexus_Send_Buffer::activate(
    Nexus_Size				size)
{
    nexus_buffer_init(&buffer,
		      (int) size,
		      0);

    object_state = OBJECT_STATE_ACTIVE;
    Nexus::status = Nexus::STATUS_SUCCESS;
    return true;
}


Nexus_Bool
Nexus_Send_Buffer::put_Byte(
    Nexus_Byte * 			array,
    unsigned				count)
{
    nexus_put_byte(&buffer, array, count);
    Nexus::status = Nexus::STATUS_SUCCESS;
    return true;
}


Nexus_Bool
Nexus_Send_Buffer::put_Char(
    char *	 			array,
    unsigned				count)
{
    nexus_put_char(&buffer, array, count);
    Nexus::status = Nexus::STATUS_SUCCESS;
    return true;
}


Nexus_Bool
Nexus_Send_Buffer::put_Int(
    int *				array,
    unsigned				count)
{
    nexus_put_int(&buffer, array, count);
    Nexus::status = Nexus::STATUS_SUCCESS;
    return true;
}


Nexus_Bool
Nexus_Send_Buffer::put_Startpoint(
    Nexus_Startpoint *			startpoint,
    unsigned				count)
{
    int					i;
    
    for (i = 0; i < count; i++)
    {
	startpoint[i].put_Internal_Startpoint(&buffer);
    }

    Nexus::status = Nexus::STATUS_SUCCESS;
    return true;
}


Nexus_Bool
Nexus_Send_Buffer::deactivate()
{
    if (object_state == OBJECT_STATE_ACTIVE)
    {
	nexus_buffer_destroy(&buffer);
    }

    Nexus::status = Nexus::STATUS_SUCCESS;
    return true;
}


Nexus_Send_Buffer::~Nexus_Send_Buffer()
{
    deactivate();
    
    object_state = OBJECT_STATE_DELETED;
    Nexus::status = Nexus::STATUS_SUCCESS;
}


Nexus_Bool
Nexus_Send_Buffer::send(
    nexus_startpoint_t *		sp_ptr,
    int					hid)
{
    int					rc;

    rc = nexus_send_rsr(&buffer,
                        sp_ptr,
                        hid,
                        NEXUS_TRUE,
                        NEXUS_FALSE);

    if (rc == NEXUS_SUCCESS)
    {
	object_state = OBJECT_STATE_INACTIVE;
	Nexus::status = Nexus::STATUS_SUCCESS;
	return true;
    }
    else
    {
	Nexus::status = Nexus::STATUS_FAILURE;
	return false;	
    }
}


